#include <stdint.h>
#include <memory.h>
#include <multiboot.h>
#include <framebuffer.h>

gfx_context kernel_fb;
gfx_context subcontext;


void putpixel(gfx_context *ctx, uint64_t x, uint64_t y, uint32_t clr)
{
    if(x >= ctx->width || y >= ctx->height) return;
    uint64_t loc = \
        y * ctx->pitch + \
        x * ctx->bpp;
    uint32_t *fb = incptr(P2V(ctx->addr), loc);

    *fb = clr;
}



gfx_context *framebuffer_make_subcontext(gfx_context *ctx, uint64_t x, uint64_t y, uint64_t width, uint64_t height)
{
    gfx_context *out = &subcontext;
    uint64_t loc = \
        y * ctx->pitch + \
        x * ctx->bpp;

    out->width = width;
    out->height = height;
    out->bpp = ctx->bpp;
    out->pitch = ctx->pitch;
    out->addr = incptr(ctx->addr, loc);
    out->size = ctx->pitch * (height);

    return out;
}

void framebuffer_init(struct fbinfo *fbinfo)
{
    kernel_fb.width = fbinfo->framebuffer_width;
    kernel_fb.height = fbinfo->framebuffer_height;
    kernel_fb.bpp = fbinfo->framebuffer_bpp/8;
    kernel_fb.pitch = fbinfo->framebuffer_pitch;
    kernel_fb.addr = (void *)fbinfo->framebuffer_addr;
    kernel_fb.size = kernel_fb.pitch * (kernel_fb.height);

    for(uintptr_t p = (uintptr_t)kernel_fb.addr; p < ((uintptr_t)kernel_fb.addr + kernel_fb.size); p += PAGE_SIZE)
    {
        vmm_set_page(kernel_P4, (uint64_t)P2V(p), p, PAGE_WRITE | PAGE_PRESENT);
    }
}