#include <stdint.h>
#include <terminal.h>
#include <memory.h>

#define FRAMEBUFFER 0x1
#define EGA_TEXT 0x2

#define VGA_ROW(pos) ((pos)/VGA_COLS)
#define VGA_COL(pos) ((pos)%VGA_COLS)
#define VGA_POS(row, col) ((row)*VGA_COLS + (col))

static struct vga_cell buffer[VGA_SIZE];
unsigned int cursor = 0;
uint8_t format = 0x7;

static int terminal_type;

void terminal_init(){
    struct fbinfo *fbinfo = kernel_boot_data.fbinfo;
    terminal_type = fbinfo->framebuffer_type;

    switch(terminal_type)
    {
        case FRAMEBUFFER:
            framebuffer_init(fbinfo);
            fbterm_init(&kernel_fb);
            break;
        case EGA_TEXT:
            vga_init(fbinfo);
            break;
    }
}


static void scroll()
{
    while(cursor >= VGA_SIZE)
    {
        memmove(buffer, &buffer[VGA_POS(1,0)], VGA_COLS*(VGA_ROWS-1)*sizeof(struct vga_cell));
        memset(&buffer[VGA_POS(VGA_ROWS-1, 0)], 0, VGA_COLS*sizeof(struct vga_cell));
        cursor -= VGA_COLS;
    }
}

void terminal_write(char c)
{
    int doflush = 0;
    switch(c)
    {
        case '\n':
            cursor += VGA_COLS - VGA_COL(cursor);
            doflush = 1;
            break;
        default:
            buffer[cursor++] = (struct vga_cell){.c = c, .f = format};
    }
    scroll();
    switch(terminal_type)
    {
        case FRAMEBUFFER:
            if(doflush)
                fbterm_flush(buffer);
            fbterm_movecursor(cursor);
            break;
        case EGA_TEXT:
            if(doflush)
                vga_flush(buffer);
            vga_movecursor(cursor);
            break;
    }
}